// ****************************************************************************
// Robert Tyrakowski
// DK7NT
// 23.06.2007

// Zu erledigen:
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//                           MAIN for RFMTX
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//  Target: MSP430F1232
//  Clock2: 32Khz
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

// MSP430F1232:
//------------
// SPI_CLK			 : Out Pin P1.0      ----------->RFM CLK
// SPI_CE		 	 : Out Pin P1.1      ----------->RFM NSEL
//      			 : In  Pin P1.2
// SPI_OUT			 : Out Pin P1.3      ----------->RFM SDI
// JTAG				 : Out Pin P1.4
// JTAG			     : Out Pin P1.5
// JTAG			     : Out Pin P1.6
// JTAG			     : Out Pin P1.7

// ACLK   		    *: Out Pin P2.0
// 					 : In  Pin P2.1
// SELP 			 : Out Pin P2.2
// TT0	             : Out Pin P2.3
// DPIIRQ 			 : In  Pin P2.4 IRQ   <-----------RFM NIRQ
// FSK   			 : OUT Pin P2.5       ----------->RFM FSK

// SF0               : In  Pin P3.0
// SF1   			 : In  Pin P3.1
// SP0      	 	 : In  Pin P3.2
// SP1				 : In  Pin P3.3
//   				 : In  Pin P3.4
//   				 : In  Pin P3.5
// ST0  			 : In  Pin P3.6
// ST1  			 : In  Pin P3.7
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Frequency selection:
//
//           SF1     SF0
//            0       0       433.400 MHz Chan 14
//            0       1       433.900 MHz Chan 30
//            1       0       434.200 MHz Chan 46
//            1       1       434.650 MHz Chan 64
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Power selection:
//
//           SP1     SP0
//            0       0       min
//            0       1
//            1       0
//            1       1       max

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Repeat time selection:
//
//           ST1     ST0
//            0       0       10 sec
//            0       1       30 sec
//            1       0       60 sec
//            1       1       120 sec

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++

#include <msp430def.h>
#include <stdio.h>
#include <msp430x12x2.h>
#include <main.h>

#include <rfmtx.h>

//--------------------------------------
unsigned char seccnt;
unsigned int currtime;
unsigned char start;

//--------------------------------------
extern void RF02_txon(void);
extern void RF02_txoff(void);
extern void rf02_sleep(void);

//----------------------------------------------------------------------------
// prototypes


void port_init()
{
 P1OUT=0xFF;
 P1DIR=0xFB;
 P1IES=0x00;
 P1IE=0x00;
 P1SEL=0x00;
 P2OUT=0x12;
 P2DIR=0x2D;
 P2IES=0x00;
 P2IE=0x00;
 P2SEL=0x03;
 P3OUT=0xFF;
 P3DIR=0x00;
 P3SEL=0x00;
}

void clock_init()
{
  //DCO=0x00
 DCOCTL=0x00;
 BCSCTL2=0xC0;//SELM.1,SELM.0
 BCSCTL1=0x00;
}

void timera_init()
//Clock source=ACLK
//Divider=1
//TIMACLK=32,768KHz
//Frequency=0,5Hz (  2,00S)
{
 TACTL=0x00; //stop timer
 //channel0
 TACCTL0=0x10;//TACCIE0
 TACCR0=0xFFFF;
 //channel1
 TACCTL1=0x00;
 TACCR1=0x00;
 //channel2
 TACCTL2=0x00;
 TACCR2=0x00;
 TAR=0x0000;//clear timer counter
 TACTL=0x124;//TASSEL.0,TAMC.1,1
}

//call this routine to initialise all peripherals
void Init_MSP430(void)
{
 //stop errant interrupts until set up
 DINT();   //disable all interrupts
 ME2=0X00; //disable sfr peripherals
 IE1=0x00; //disable sfr interrupts
 IE2=0x00;

 //watchdog initialisation including nmi function
 WDTCTL=0x5A00 | 0x80;

 //initialise other peripherals
 clock_init();
 port_init();
 timera_init();

 ME2=0x00;
 IE1=0x00;
 IE2=0x00;
 EINT(); //re-enable interrupts
}

//interrupt handlers

//            0       0       10 sec
//            0       1       30 sec
//            1       0       60 sec
//            1       1       120 sec

#pragma interrupt_handler timera_0_isr:TIMERA0_VECTOR
void timera_0_isr(void)
{
unsigned char z;
   seccnt++;
   switch (currtime) {
      case 0: z = ct0;
	  		 	 break;
	   case 1: z = ct1;
	  		 	 break;
      case 2: z = ct2;
	  		 	 break;
	   case 3: z = ct3;
	  		 	 break;
	   default : ;
    }
	 if (seccnt++ >= z) {
      LPM3_EXIT;
		seccnt = 0;
  }
}

//General short delay
void delay(unsigned int x)
{
unsigned int i;
 for (i=0; i<x; i++)
  ;
}

//
void 	  Enter_PD(void)
{
 		P1OUT=0x00;
 		P1DIR=0x00;
 		P1IES=0x00;
 		P1IE=0x00;
 		P1SEL=0x00;
 		P2OUT=0x00;
 		P2DIR=0x00;
 		P2IES=0x00;
 		P2IE=0x00;
 		P2SEL=0x00;
 		P3OUT=0x00;
 		P3DIR=0x00;
 		P3SEL=0x00;
        rf02_sleep();
  	    LPM3;	 	   				// enter LPM3
}

void Blink_LED(void)
{
unsigned char i;
   LED_ON ;
   for (i=0; i<20; i++)
         delay(3);
	LED_OFF;
}

void main()
{
unsigned char x,dummy;
unsigned int i;
unsigned int currfreq,currpwr;
float ssff;
   Init_MSP430();
   Blink_LED();
   SELP_ON;		  					 // get settings
   currfreq = SF ;
   currpwr = SP ;
   currtime = ST ;
   SELP_OFF;

   rf02_init();	            // ein paar Register setzen (z.B. CLK auf 10MHz)
	RF02_txoff();
   switch (currfreq) {
      case 0: ssff = ssf0;
	  	   break;
      case 1: ssff = ssf1;
	  	   break;
      case 2: ssff = ssf2;
	  	   break;
      case 3: ssff = ssf3;
	  	   break;
	  default: ;
   }
   rf02_setfreq(RF02FREQ(ssff));	// Sendefrequenz einstellen
   switch (currpwr) {
      case 0: i = ssp0;
	  	   break;
      case 1: i = ssp1;
	  	   break;
      case 2: i = ssp2;
	  	   break;
      case 3: i = ssp3;
	  	   break;
	  default: ;
   }
   rf02_setpower(i);				// Ausgangangsleistung
   rf02_setmodfreq(0);			// 30kHz Frequenzshift
//   rf02_setbaud(1345);		// 1345 Baud
   seccnt = 0;
	start = 1;
   for (i=0; i<200; i++)	   // time to allow Debugger entry
            delay(100);
	Blink_LED();
	while(1)	{
      LED_OFF;
// prepare enter Power Down Mode
	   if (!start)
		   Enter_PD();
      if ( !seccnt || start ) {
      /* transmit packet */
		   port_init();
			Blink_LED();
	      RF02_txon();		 		   // tx on
			for (i=0; i<10; i++)			// wait for PLL stabile
	        delay(10);
	      FSK_OFF;
         for (i=0; i<20; i++)	   	   // send carrier 1
            delay(100);
         RF02_txoff();		 	       // tx off
         for (i=0; i<20; i++)		   // wait
            delay(10);
	      RF02_txon();		 			// tx on
	      FSK_OFF;
         for (i=0; i<60; i++)		   // send carrier 2
            delay(10);
			for (i=0; i<0x200;i++) {   // send modulation 1
		      FSK_ON;
		      FSK_OFF;
		      FSK_OFF;
         }
         for (i=0; i<20; i++)		   // send carrier 3
            delay(10);
			for (i=0; i<0x200;i++) {   // send modulation 2
				FSK_ON;
				FSK_OFF;
				FSK_OFF;
			}
         for (i=0; i<20; i++)		   // send carrier 4
            delay(10);
			for (i=0; i<0x200;i++) {   // send modulation 3
				FSK_ON;
				FSK_OFF;
				FSK_OFF;
			}
	      RF02_txoff();
			seccnt = 1;
			start = 0;
		}
   }
}
