// ****************************************************************************
// Robert Tyrakowski
// RFMTX.C
// 15.05.2007
// ****************************************************************************

#include <msp430def.h>
#include <stdio.h>
#include <msp430x12x2.h>

//----------------------------------------------------------------------------
// prototypes

//------------------------------------------------------------------------------
//RFM02      - Transmitter
//------------------------------------------------------------------------------
//#include "global.h"
#include "rfmtx.h"

// SPI_CLK			 : Out Pin P1.0
// SPI_CE		 	 : Out Pin P1.1
// SPI_IN 			 : In  Pin P1.2
// SPI_OUT			 : Out Pin P1.3

#define SPIOUTLOW P1OUT = P1IN & ~BIT3
#define SPIOUTHIGH P1OUT = P1IN | BIT3

#define CE_HIGH   P1OUT = P1IN |= BIT1
#define CE_LOW    P1OUT = P1IN &= ~BIT1

#define CLK_HIGH  P1OUT = P1IN | BIT0
#define CLK_LOW   P1OUT = P1IN & ~BIT0

#define IRQ_IN    (P2IN & BIT4) >> 4

extern void delay(unsigned int x);

// send 16 bit to MRF02

void rf02_trans(unsigned short wert)
{	unsigned char i;

	CE_LOW;
	for (i=0; i<16; i++)
	{	if (wert & 0x8000)
			SPIOUTHIGH;
		else
			SPIOUTLOW;
		CLK_HIGH;
		wert<<=1;
	 	delay(1);
		CLK_LOW;
	}
	CE_HIGH;
}

void rf02_init(void)
{
unsigned char i;
	CE_HIGH;
	CLK_LOW;
	for (i=0; i<1; i++)
	   delay(1);
//	rf02_trans(0xC000 | oscena | synena | paamp  );
   rf02_trans(0xC000);
//	rf02_trans(0xC000 | autoosc | autopa  );
	rf02_trans(0x8000 | F433MHz | CLKO_02_00MHZ | CLOAD_08_5);
//	rf02_trans(0xC2A0);			// enable tx sync bit, disable low bat detector
}

void rf02_setmodfreq(unsigned char bandwidth)
{
	rf02_trans(0x8000 | F433MHz | CLKO_02_00MHZ | CLOAD_08_5 |(bandwidth&7));
}

void rf02_setfreq(unsigned short freq)
{
	if (freq<96)				   // 430,2400MHz
		freq=96;
	else if (freq>3903)			   // 439,7575MHz
		freq=3903;
	rf02_trans(0xA000|freq);
}

void rf02_setpower(unsigned char power)
{
	rf02_trans(0xB000|((power&7)<<8));
}

void rf02_sleep(void)
{
   rf02_trans(0xC400);
}

void rf02_setbaud(unsigned short baud)
{
	if (baud<1345)
		baud=1345;
	if (baud<19000)
		rf02_trans(0xD240);		// 25% PLL current
	else if (baud<37000)
		rf02_trans(0xD2C0);		// 33% PLL current
	else
		rf02_trans(0xD200);		// 50% PLL current
	rf02_trans(0xC800|((344828UL/baud)-1));	// Baudrate= 344827,59/(R+1)
}

void rf02_txdata(unsigned char *data, unsigned char number)
{	unsigned char i,j,wert;
	wert=0xC6;
	CE_LOW;
	for (i=0; i<8; i++)	{
   	if (wert & 0x80)
			SPIOUTHIGH;
		else
			SPIOUTLOW;
		CLK_HIGH;
		wert<<=1;
		CLK_LOW;
	}
	rf02_shiftout(0xAA);
	rf02_shiftout(0xAA);
	rf02_shiftout(0xAA);
	rf02_shiftout(0x2D);
	rf02_shiftout(0xD4);
	for (i=0; i<number; i++)
		rf02_shiftout(*data++);
	CE_HIGH;
	while(IRQ_IN);		// wait until transfer done
	rf02_trans(0xC464);			// TX off after 10us
}

// Shift 8 data bit out during Transmition

void rf02_shiftout(unsigned char wert)
{	unsigned char j;
	for (j=0; j<8; j++)
	{	while(IRQ_IN);
    	while(!(IRQ_IN));
		if (wert & 0x80)
    		SPIOUTHIGH;
    	else
    		SPIOUTLOW;
    	wert<<=1;
		CLK_HIGH;
		CLK_LOW;

    }
}

void RF02_txon(void)
{
	rf02_trans(0xC000 | oscena | synena | paamp |clkodis );
}

void RF02_txoff(void)
{
	rf02_trans(0xC000 );
}

